import django_filters
from django_filters import FilterSet
from .models import Emoji


class EmojiFilter(FilterSet):
    name = django_filters.CharFilter(
        field_name="name",
        lookup_expr="icontains",
        label="Search by emoticon name",
    )
    team = django_filters.CharFilter(
        field_name="team__name",
        lookup_expr="icontains",
        label="Search by team name",
    )
    creator = django_filters.CharFilter(
        field_name="created_by__username",
        lookup_expr="iexact",
        label="Search by creator username",
    )

    class Meta:
        model = Emoji
        fields = ["name", "nsfw", "team", "private", "creator"]



"""Permission classes"""

from rest_framework import permissions


class IsOwnerOrReadOnly:
    """
    Custom permission to only allow owners of an object to edit it.
    """

    def has_permission(self, request, view):
        # Read permissions are allowed to any request,
        # so we'll always allow GET, HEAD or OPTIONS requests.
        if request.method in permissions.SAFE_METHODS:
            return True

        return request.user.is_authenticated

    def has_object_permission(self, request, view, obj):
        # Read permissions are allowed to any request,
        # so we'll always allow GET, HEAD or OPTIONS requests.
        if request.method in permissions.SAFE_METHODS:
            return True

        # Write permissions are only allowed to the owner of the snippet.
        return obj.created_by == request.user



from rest_framework.routers import Route, SimpleRouter


class EmojiRouter(SimpleRouter):
    """
    A router for read-only APIs, which doesn't use trailing slashes.
    """

    routes = [
        Route(
            url=r"^{prefix}s$",
            mapping={"get": "list"},
            name="{basename}-list",
            detail=False,
            initkwargs={"suffix": "List"},
        ),
        Route(
            url=r"^{prefix}/{lookup}$",
            mapping={
                "get": "retrieve",
                "post": "create",
                "delete": "destroy",
            },
            name="{basename}-detail",
            detail=True,
            initkwargs={"suffix": "Detail"},
        ),
    ]



from django_unicorn.components import UnicornView


class SearchView(UnicornView):
    search = ""

    def updated_search(self, query):
        self.parent.load_emojis(search=query)
        self.parent.force_render = True



"""Main page."""
from django.shortcuts import render
from django.views.generic import TemplateView


class IndexView(TemplateView):
    template_name = "index.html"

    def get(self, request, *args, **kwargs):
        context = {
            "title": "Emoticons",
        }
        return render(request, self.template_name, context)



"""Emoticons URLs."""

from django.conf.urls import include
from django.urls import path

from .viewsets import EmojiViewSet
from .routers import EmojiRouter
from .views import IndexView

app_name = "emojis"

router = EmojiRouter(trailing_slash=False)
router.register("emoticon", EmojiViewSet, basename="emoticons")

urlpatterns = [
    path("", include(router.urls)),
    path("", IndexView.as_view(), name="index"),
]



"Emoji model"
from django.db import models
from django.contrib.auth.models import User, Group


class EmojiAdminManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset()

    def sfw(self, user):
        qs = self.nsfw(user=user)
        qs = qs.filter(nsfw=False)
        return qs

    def nsfw(self, user):
        qs = self.get_queryset()
        if user.is_authenticated:
            qs = qs.filter(models.Q(created_by=user) | models.Q(private=False))
        else:
            qs = qs.filter(private=False)
        return qs


class EmojiManager(EmojiAdminManager):
    def get_queryset(self):
        return (
            super()
            .get_queryset()
            .filter(deleted_at__isnull=True, image__isnull=False)
        )


class EmojiUpdateManager(EmojiAdminManager):
    def get_queryset(self):
        return super().get_queryset().filter(deleted_at__isnull=True)


class Emoji(models.Model):
    EXPORT_FIELDS = [
        "name",
        "team",
        "private",
        "uses",
        "nsfw",
        "created_at",
        "created_by",
        "deleted_at",
        "deleted_by",
    ]

    name = models.CharField(max_length=255, primary_key=True)
    uses = models.IntegerField(default=0)
    image = models.BinaryField(null=True)
    team = models.ForeignKey(
        Group,
        related_name="emojis",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
    )
    nsfw = models.BooleanField(default=False)
    private = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    created_by = models.ForeignKey(
        User, related_name="created_emojis", on_delete=models.CASCADE
    )
    deleted_at = models.DateTimeField(null=True, blank=True)
    deleted_by = models.ForeignKey(
        User,
        related_name="deleted_emojis",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
    )
    objects = EmojiManager()
    updates = EmojiUpdateManager()
    admin = models.Manager()



# Generated by Django 5.0 on 2023-12-22 16:47

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("emodjis", "0002_emoji_nsfw_emoji_team"),
    ]

    operations = [
        migrations.AddField(
            model_name="emoji",
            name="private",
            field=models.BooleanField(default=False),
        ),
    ]
